#!/bin/bash
# this script is used to install/uninstall HF or IB
# date: 12-12-05
#set -eu -o pipefail

# check user account, must be done before ExitTrap
if [ `whoami` != "root" ]
then
	echo "Error[126] Line[${LINENO}]: This hotfix must be installed by super user root."
	exit 126
fi

FIPS_enabled=0
UninstallFlag=0
FIPS_FILE_LIST=""
CG_DIR=`cat /etc/content_gateway` || CG_DIR="/opt/WCG"
PATCH_DIR=`dirname $(readlink /proc/$$/fd/255)` || PATCH_DIR=$PWD
BACKUP_DIR="${CG_DIR}/hotfix"
LOG_FILE="${CG_DIR}/logs/hotfix_install.log"
NOW_TIME=`date +%y%m%d` || NOW_TIME="000000"

archive="WCG_7.8.4_Hotfix_09_WCG_Upgrade_OpenSSL_To_1_0_1_m.tar.gz" # customize this archive name 
serialNum=0					# serial number of hotfix, only defined in this script, do not modify
fileList=""					# files to be replaced
fileAccs=()
fileOwns=()

# print input to both screen and log file
function TeeLog
{
	while read content || exit 0		# when no input, just return
	do
		echo ${content}
		echo "[$(date)] ${content}" >> ${LOG_FILE}
	done
}

# when exit, check exit code
function ExitTrap
{
	exitCode=$?
	if [ ${exitCode} -ne 0 ]
	then
		echo -e "Error[${exitCode}] Line[$1].\n" | TeeLog
		echo "For details, see hotfix.log."
	fi
	exit ${exitCode}
}
trap 'ExitTrap ${LINENO}' EXIT

# backup files and record their access and ownership
function PrepareHF
{
	echo "Prepare hotfix..." | TeeLog
	trap '${CG_DIR}/WCGAdmin start' ERR
	trap '${CG_DIR}/WCGAdmin start; exit 130' 2

	# serialNum should be greater than any serialNum of HF installed before
	serialNum=`ls ${BACKUP_DIR}|gawk -F_ '/^[[:digit:]]+_[[:digit:]]+_.+\.tar.gz$/{print $1}'|sort -g|tail -n 1`
	if [ -z ${serialNum} ]						# this is the first HF on this machine
	then
		serialNum=1
	else
		serialNum=$[${serialNum} + 1]
	fi
	tar ztf ${PATCH_DIR}/${archive} 2>&1 1>/dev/null | TeeLog	# check archive firstly
	FIPS_FILE_LIST=`tar ztf ${PATCH_DIR}/${archive} |grep '^FIPS'|grep -v '/$'`
	FIPS_enabled_tmp=`/bin/cat ${CG_DIR}/config/records.config|grep "CONFIG proxy.config.fips.security_enabled INT" `
  	if [ -n "$FIPS_enabled_tmp" ]; then
        	FIPS_enabled=`/bin/cat ${CG_DIR}/config/records.config|grep "CONFIG proxy.config.fips.security_enabled INT"|awk '{print $4}'`
  	fi
	if [ -n "$FIPS_FILE_LIST" ] && [ $FIPS_enabled -eq 1 ] ;then
		fileList=`tar ztf ${PATCH_DIR}/${archive} |grep '^FIPS'|grep -v '/$'|awk -F "FIPS/" '{print $2}'`
	else		
		fileList=`tar ztf ${PATCH_DIR}/${archive} | grep -v '^FIPS' | grep -v '/$'`	# gen file list exclude directories
	fi
	
	echo "Stopping Websense Content Gateway..." | TeeLog
	${CG_DIR}/WCGAdmin stop &> /dev/null

	echo "Check origin files and record their access and ownership..." | TeeLog
	fn=0
	for file in ${fileList}
	do
		stat -c "%a %U %G %n" ${CG_DIR}/${file} 2>&1 | TeeLog
		md5sum ${CG_DIR}/${file} 2>&1 | TeeLog
		fileAccs[$fn]=`stat -c %a ${CG_DIR}/${file}`
		fileOwns[$fn]=`stat -c %U:%G ${CG_DIR}/${file}`
		fn=$[$fn + 1]
	done

	echo "Backup files to folder ${BACKUP_DIR}..." | TeeLog
	cd ${CG_DIR}							# make sure the backup file include proper path
	tar zcf ${BACKUP_DIR}/${serialNum}_${NOW_TIME}_${archive} ${fileList} 2>&1 | TeeLog

#	trap - ERR 2
}

# when error occur during InstallHF, rollback
function RollbackHF
{
	echo "Rollback hotfix..." | TeeLog
	tar zxf ${BACKUP_DIR}/${serialNum}_${NOW_TIME}_${archive} -C ${CG_DIR} 2>&1 | TeeLog
	echo "Starting content gateway..." | TeeLog
	${CG_DIR}/WCGAdmin start &> /dev/null
}

# after PrepareHF, begin to install
function InstallHF
{
	echo "Replace files..." | TeeLog
	trap 'RollbackHF' ERR
	trap 'RollbackHF; exit 130' 2
	if [ $UninstallFlag  -eq 0 ]; then
	
		tar zxf ${PATCH_DIR}/${archive} -C  ${PATCH_DIR} 2>&1 | TeeLog	# replace files
		fn=0
		for file in ${fileList}
		do
			if [ -n "$FIPS_FILE_LIST" ] && [ $FIPS_enabled -eq 1 ] ; then
				/bin/cp -arf ${PATCH_DIR}/FIPS/${file} ${CG_DIR}/${file} 2>&1 | TeeLog
			else
				/bin/cp -afr ${PATCH_DIR}/${file} ${CG_DIR}/${file} 2>&1 | TeeLog
			fi
		done
	else 
		tar zxf  ${PATCH_DIR}/${archive} -C  ${CG_DIR} 2>&1 | TeeLog 
	fi

	fn=0
	for file in ${fileList}
	do
		chown -v --no-dereference ${fileOwns[$fn]} ${CG_DIR}/${file} 2>&1 | TeeLog
		chmod -v ${fileAccs[$fn]} ${CG_DIR}/${file} 2>&1 | TeeLog
		md5sum ${CG_DIR}/${file} 2>&1 | TeeLog
		fn=$[$fn + 1]
	done

	echo "Starting content gateway..." | TeeLog
	${CG_DIR}/WCGAdmin start &> /dev/null
#	trap - ERR 2
}

mkdir -vp ${BACKUP_DIR} 2>&1 | TeeLog				# make sure the two directories existed, if already existed, do nothing
mkdir -vp ${CG_DIR}/logs 2>&1 | TeeLog
if [ $# -eq 0 ]
then
	echo "Begin installing ${archive}..." | TeeLog
	PrepareHF
	InstallHF
	echo -e "${archive} was successfully installed!\n" | TeeLog
elif [ $1 = "rollback" ]
then
	# get archive name with the largest serialNum under backup directory
	archive=`ls ${BACKUP_DIR}|gawk '/^[[:digit:]]+_[[:digit:]]+_.+\.tar.gz$/'|sort -g|tail -n 1`
	if [ -z ${archive} ]
	then
		echo -e "Cannot find any backup on this machine, nothing to do.\n" | TeeLog
		exit 1
	fi
	mv -vf ${BACKUP_DIR}/${archive} ${PATCH_DIR} 2>&1 | TeeLog
	echo "Begin uninstalling ${archive}..." | TeeLog
	UninstallFlag=1    #If it is uninstallation, set it to 1
	PrepareHF
	InstallHF
	rm -vf ${PATCH_DIR}/${archive} 2>&1 | TeeLog
	mv -vf ${BACKUP_DIR}/${serialNum}_${NOW_TIME}_${archive} ${BACKUP_DIR}/u${archive} 2>&1 | TeeLog
	echo -e "${archive} was successfully uninstalled!\n" | TeeLog
else
	echo "Usage:"
	echo -e "\t1. Install  : this script will install hotfix/IB by default."
	echo -e "\t2. Rollback : run this script with parameter 'rollback'.\n"
fi
exit 0
